Literate

Struct Literate 

pub struct Literate<I> { /* private fields */ }
Adds Display/Debug formatting to any iterator whose items implement Display and/or Debug.

All formatting arguments are passed through to each item’s Display/Debug implementation.

Implementations

Source

impl<I: Sized> Literate<I>

pub const fn new(iter: I) -> Self

Wrap an iterator in a Literate struct.

pub const fn new_cell(iter: Option<I>) -> Self

Wrap an optional iterator in a Literate struct.

pub fn into_inner(self) -> Option<I>

Consumes the Literate and returns the inner iterator. Returns None if the object has been formatted already.

pub fn map_iterator<F, O>(self, f: F) -> Literate<O>
where F: FnOnce(I) -> O,

Convert the inner iterator to another inner iterator.